/** 
* @file ColorTable32KMod.cpp
* Implementation of class ColorTable32KMod.
*
* @author <A href=mailto:walter.nistico@uni-dortmund.de>Walter Nistico</A>
*/

#include "ColorTable32KMod.h"
#include "Tools/Location.h"

/* 
I don't like this.
Directly copying an object to a memory buffer is a C-style low level way of instantiating it, 
beside, the initialization of the memory buffer looks quite ugly because of the test operators
to determine the size of the biggest colortable object, and it gets worser and worser as new
colortables are implemented.
I have a more elegant solution which gives access to colortables to the image processors 
through a colortable pointer: it works, but it needed a thorough rewriting of the image processor
interface, and it has a potential for problems with image processors designed to use a 
single color table (ColorTable64) such as GridImageProcessor2.
If you want to support this second approach, please write me an e-mail.
*/
void ColorTable32KMod::setup()
{
  ColorTable32K* colorTable32K = new ColorTable32K();
  InBinaryFile stream(getLocation().getFilename("coltable.c32"));
  if (stream.exists())
  {
    stream >> *colorTable32K;
  }
  memcpy(colorTable, colorTable32K, sizeof(ColorTable32K));
  delete colorTable32K;
}

/*
* Change log :
* 
* $Log: ColorTable32KMod.cpp,v $
* Revision 1.1  2004/01/30 15:44:34  nistico
* Created a Color Table Selector, which permits to switch color tables at run time using the same image processor.
* If your image processor was designed to use only ColorTable64, leave it as the default choice and you should
* (hopefully) have no problems.
* In case of trouble, i put a conditional compilation switch in SolutionRequest.h, simply uncomment the definition of NO_COLOR_TABLE_SELECTOR; in that case, the code should get back to the original solution, but the coltable which will be looked upon first for loading will be ColorTable64 (as it's the default solution), so there's no reason anymore to remove the other color tables from the CVS.
*
*
*/
